begin
  require 'retryable'
rescue LoadError
  STDERR.puts 'BusinessFlow::Retryable requires the retryable gem'
  raise
end

module BusinessFlow
  # Wraps a flow in a Retryable (https://github.com/nfedyashev/retryable)
  # You must provide this gem yourself
  module Retryable
    def self.included(klass)
      klass.extend(ClassMethods)
    end

    # Additions to the DSL
    module ClassMethods
      def retryable(opts)
        @retryable_config ||= {}
        @retryable_config.merge!(opts)
      end

      def execute(flow)
        ::Retryable.retryable(@retryable_config) do
          super(flow)
        end
      end
    end
  end
end
