module BusinessFlow
  # Responsible for adding validations to flow objects
  module Validations
    # Additions to the DSL
    module ClassMethods
      # Declares that you expect to set this field during the course of
      # processing, and that it should meet the given ActiveModel
      # validations.
      def expects(field, options = {})
        validates field, options
        ::BusinessFlow::DSL::Field.new(field).add_to(self)
      end
    end

    def self.included(klass)
      klass.include(ActiveModel::Validations)
      klass.class_eval do
        class << self
          alias_method :invariant, :validates
        end
      end
      klass.extend ClassMethods
    end
  end
end
